@@ -131,7 +131,8 @@ void BB2DomainInfo::setCtime()
   d_ctime = buf.st_ctime;
 }
 
-bool Bind2Backend::safeGetBBDomainInfo(int id, BB2DomainInfo* bbd)
+// NOLINTNEXTLINE(readability-identifier-length)
+bool Bind2Backend::safeGetBBDomainInfo(domainid_t id, BB2DomainInfo* bbd)
 {
   auto state = s_state.read_lock();
   state_t::const_iterator iter = state->find(id);
@@ -142,7 +143,7 @@ bool Bind2Backend::safeGetBBDomainInfo(int id, BB2DomainInfo* bbd)
   return true;
 }
 
-bool Bind2Backend::safeGetBBDomainInfo(const DNSName& name, BB2DomainInfo* bbd)
+bool Bind2Backend::safeGetBBDomainInfo(const ZoneName& name, BB2DomainInfo* bbd)
 {
   auto state = s_state.read_lock();
   const auto& nameindex = boost::multi_index::get<NameTag>(*state);
@@ -154,7 +155,7 @@ bool Bind2Backend::safeGetBBDomainInfo(const DNSName& name, BB2DomainInfo* bbd)
   return true;
 }
 
-bool Bind2Backend::safeRemoveBBDomainInfo(const DNSName& name)
+bool Bind2Backend::safeRemoveBBDomainInfo(const ZoneName& name)
 {
   auto state = s_state.write_lock();
   using nameindex_t = state_t::index<NameTag>::type;
@@ -174,7 +175,8 @@ void Bind2Backend::safePutBBDomainInfo(const BB2DomainInfo& bbd)
   replacing_insert(*state, bbd);
 }
 
-void Bind2Backend::setNotified(uint32_t id, uint32_t serial)
+// NOLINTNEXTLINE(readability-identifier-length)
+void Bind2Backend::setNotified(domainid_t id, uint32_t serial)
 {
   BB2DomainInfo bbd;
   if (!safeGetBBDomainInfo(id, &bbd))
@@ -183,7 +185,8 @@ void Bind2Backend::setNotified(uint32_t id, uint32_t serial)
   safePutBBDomainInfo(bbd);
 }
 
-void Bind2Backend::setLastCheck(uint32_t domain_id, time_t lastcheck)
+// NOLINTNEXTLINE(readability-identifier-length)
+void Bind2Backend::setLastCheck(domainid_t domain_id, time_t lastcheck)
 {
   BB2DomainInfo bbd;
   if (safeGetBBDomainInfo(domain_id, &bbd)) {
@@ -192,31 +195,32 @@ void Bind2Backend::setLastCheck(uint32_t domain_id, time_t lastcheck)
   }
 }
 
-void Bind2Backend::setStale(uint32_t domain_id)
+void Bind2Backend::setStale(domainid_t domain_id)
 {
-  setLastCheck(domain_id, 0);
+  Bind2Backend::setLastCheck(domain_id, 0);
 }
 
-void Bind2Backend::setFresh(uint32_t domain_id)
+void Bind2Backend::setFresh(domainid_t domain_id)
 {
-  setLastCheck(domain_id, time(nullptr));
+  Bind2Backend::setLastCheck(domain_id, time(nullptr));
 }
 
-bool Bind2Backend::startTransaction(const DNSName& qname, int id)
+bool Bind2Backend::startTransaction(const ZoneName& qname, domainid_t domainId)
 {
-  if (id < 0) {
+  if (domainId == UnknownDomainID) {
     d_transaction_tmpname.clear();
-    d_transaction_id = id;
+    d_transaction_id = UnknownDomainID;
+    // No support for domain contents deletion
     return false;
   }
-  if (id == 0) {
+  if (domainId == 0) {
     throw DBException("domain_id 0 is invalid for this backend.");
   }
 
-  d_transaction_id = id;
+  d_transaction_id = domainId;
   d_transaction_qname = qname;
   BB2DomainInfo bbd;
-  if (safeGetBBDomainInfo(id, &bbd)) {
+  if (safeGetBBDomainInfo(domainId, &bbd)) {
     d_transaction_tmpname = bbd.d_filename + "XXXXXX";
     int fd = mkstemp(&d_transaction_tmpname.at(0));
     if (fd == -1) {
@@ -245,8 +249,11 @@ bool Bind2Backend::startTransaction(const DNSName& qname, int id)
 
 bool Bind2Backend::commitTransaction()
 {
-  if (d_transaction_id < 0)
+  // d_transaction_id is only set to a valid domain id if we are actually
+  // setting up a replacement zone file with the updated data.
+  if (d_transaction_id == UnknownDomainID) {
     return false;
+  }
   d_of.reset();
 
   BB2DomainInfo bbd;
@@ -256,28 +263,92 @@ bool Bind2Backend::commitTransaction()
     queueReloadAndStore(bbd.d_id);
   }
 
-  d_transaction_id = 0;
+  d_transaction_id = UnknownDomainID;
 
   return true;
 }
 
 bool Bind2Backend::abortTransaction()
 {
-  // -1 = dnssec speciality
-  // 0  = invalid transact
-  // >0 = actual transaction
-  if (d_transaction_id > 0) {
+  // d_transaction_id is only set to a valid domain id if we are actually
+  // setting up a replacement zone file with the updated data.
+  if (d_transaction_id != UnknownDomainID) {
     unlink(d_transaction_tmpname.c_str());
     d_of.reset();
-    d_transaction_id = 0;
+    d_transaction_id = UnknownDomainID;
   }
 
   return true;
 }
 
+static bool ciEqual(const string& lhs, const string& rhs)
+{
+  if (lhs.size() != rhs.size()) {
+    return false;
+  }
+
+  string::size_type pos = 0;
+  const string::size_type epos = lhs.size();
+  for (; pos < epos; ++pos) {
+    if (dns_tolower(lhs[pos]) != dns_tolower(rhs[pos])) {
+      return false;
+    }
+  }
+  return true;
+}
+
+/** does domain end on suffix? Is smart about "wwwds9a.nl" "ds9a.nl" not matching */
+static bool endsOn(const string& domain, const string& suffix)
+{
+  if (suffix.empty() || ciEqual(domain, suffix)) {
+    return true;
+  }
+
+  if (domain.size() <= suffix.size()) {
+    return false;
+  }
+
+  string::size_type dpos = domain.size() - suffix.size() - 1;
+  string::size_type spos = 0;
+
+  if (domain[dpos++] != '.') {
+    return false;
+  }
+
+  for (; dpos < domain.size(); ++dpos, ++spos) {
+    if (dns_tolower(domain[dpos]) != dns_tolower(suffix[spos])) {
+      return false;
+    }
+  }
+
+  return true;
+}
+
+/** strips a domain suffix from a domain, returns true if it stripped */
+static bool stripDomainSuffix(string* qname, const ZoneName& zonename)
+{
+  std::string domain = zonename.operator const DNSName&().toString();
+
+  if (!endsOn(*qname, domain)) {
+    return false;
+  }
+
+  if (toLower(*qname) == toLower(domain)) {
+    *qname = "@";
+  }
+  else {
+    if ((*qname)[qname->size() - domain.size() - 1] != '.') {
+      return false;
+    }
+
+    qname->resize(qname->size() - domain.size() - 1);
+  }
+  return true;
+}
+
 bool Bind2Backend::feedRecord(const DNSResourceRecord& rr, const DNSName& /* ordername */, bool /* ordernameIsNSEC3 */)
 {
-  if (d_transaction_id < 1) {
+  if (d_transaction_id == UnknownDomainID) {
     throw DBException("Bind2Backend::feedRecord() called outside of transaction");
   }
 
@@ -286,7 +357,7 @@ bool Bind2Backend::feedRecord(const DNSResourceRecord& rr, const DNSName& /* ord
     qname = rr.qname.toString();
   }
   else if (rr.qname.isPartOf(d_transaction_qname)) {
-    if (rr.qname == d_transaction_qname) {
+    if (rr.qname == d_transaction_qname.operator const DNSName&()) {
       qname = "@";
     }
     else {
@@ -308,7 +379,7 @@ bool Bind2Backend::feedRecord(const DNSResourceRecord& rr, const DNSName& /* ord
   case QType::CNAME:
   case QType::DNAME:
   case QType::NS:
-    stripDomainSuffix(&content, d_transaction_qname.toString());
+    stripDomainSuffix(&content, d_transaction_qname);
     // fallthrough
   default:
     if (d_of && *d_of) {
@@ -343,7 +414,7 @@ void Bind2Backend::getUpdatedPrimaries(vector<DomainInfo>& changedDomains, std::
   for (DomainInfo& di : consider) {
     soadata.serial = 0;
     try {
-      this->getSOA(di.zone, soadata); // we might not *have* a SOA yet, but this might trigger a load of it
+      this->getSOA(di.zone, di.id, soadata); // we might not *have* a SOA yet, but this might trigger a load of it
     }
     catch (...) {
       continue;
@@ -389,7 +460,7 @@ void Bind2Backend::getAllDomains(vector<DomainInfo>* domains, bool getSerial, bo
       if (di.backend != this)
         continue;
       try {
-        this->getSOA(di.zone, soadata);
+        this->getSOA(di.zone, di.id, soadata);
       }
       catch (...) {
         continue;
@@ -425,7 +496,7 @@ void Bind2Backend::getUnfreshSecondaryInfos(vector<DomainInfo>* unfreshDomains)
     soadata.refresh = 0;
     soadata.serial = 0;
     try {
-      getSOA(sd.zone, soadata); // we might not *have* a SOA yet
+      getSOA(sd.zone, sd.id, soadata); // we might not *have* a SOA yet
     }
     catch (...) {
     }
@@ -436,26 +507,26 @@ void Bind2Backend::getUnfreshSecondaryInfos(vector<DomainInfo>* unfreshDomains)
   }
 }
 
-bool Bind2Backend::getDomainInfo(const DNSName& domain, DomainInfo& di, bool getSerial)
+bool Bind2Backend::getDomainInfo(const ZoneName& domain, DomainInfo& info, bool getSerial)
 {
   BB2DomainInfo bbd;
   if (!safeGetBBDomainInfo(domain, &bbd))
     return false;
 
-  di.id = bbd.d_id;
-  di.zone = domain;
-  di.primaries = bbd.d_primaries;
-  di.last_check = bbd.d_lastcheck;
-  di.backend = this;
-  di.kind = bbd.d_kind;
-  di.serial = 0;
+  info.id = bbd.d_id;
+  info.zone = domain;
+  info.primaries = bbd.d_primaries;
+  info.last_check = bbd.d_lastcheck;
+  info.backend = this;
+  info.kind = bbd.d_kind;
+  info.serial = 0;
   if (getSerial) {
     try {
       SOAData sd;
       sd.serial = 0;
 
-      getSOA(bbd.d_name, sd); // we might not *have* a SOA yet
-      di.serial = sd.serial;
+      getSOA(bbd.d_name, bbd.d_id, sd); // we might not *have* a SOA yet
+      info.serial = sd.serial;
     }
     catch (...) {
     }
@@ -464,7 +535,7 @@ bool Bind2Backend::getDomainInfo(const DNSName& domain, DomainInfo& di, bool get
   return true;
 }
 
-void Bind2Backend::alsoNotifies(const DNSName& domain, set<string>* ips)
+void Bind2Backend::alsoNotifies(const ZoneName& domain, set<string>* ips)
 {
   // combine global list with local list
   for (const auto& i : this->alsoNotify) {
@@ -525,7 +596,7 @@ void Bind2Backend::parseZoneFile(BB2DomainInfo* bbd)
 
 /** THIS IS AN INTERNAL FUNCTION! It does moadnsparser prio impedance matching
     Much of the complication is due to the efforts to benefit from std::string reference counting copy on write semantics */
-void Bind2Backend::insertRecord(std::shared_ptr<recordstorage_t>& records, const DNSName& zoneName, const DNSName& qname, const QType& qtype, const string& content, int ttl, const std::string& hashed, bool* auth)
+void Bind2Backend::insertRecord(std::shared_ptr<recordstorage_t>& records, const ZoneName& zoneName, const DNSName& qname, const QType& qtype, const string& content, int ttl, const std::string& hashed, const bool* auth)
 {
   Bind2DNSRecord bdr;
   bdr.qname = qname;
@@ -540,8 +611,7 @@ void Bind2Backend::insertRecord(std::shared_ptr<recordstorage_t>& records, const
       g_log << Logger::Warning << msg << " ignored" << endl;
       return;
     }
-    else
-      throw PDNSException(msg);
+    throw PDNSException(std::move(msg));
   }
 
   //  bdr.qname.swap(bdr.qname);
@@ -569,7 +639,7 @@ string Bind2Backend::DLReloadNowHandler(const vector<string>& parts, Utility::pi
 
   for (auto i = parts.begin() + 1; i < parts.end(); ++i) {
     BB2DomainInfo bbd;
-    DNSName zone(*i);
+    ZoneName zone(*i);
     if (safeGetBBDomainInfo(zone, &bbd)) {
       Bind2Backend bb2;
       bb2.queueReloadAndStore(bbd.d_id);
@@ -577,7 +647,7 @@ string Bind2Backend::DLReloadNowHandler(const vector<string>& parts, Utility::pi
         ret << *i << ": [missing]\n";
       else
         ret << *i << ": " << (bbd.d_wasRejectedLastReload ? "[rejected]" : "") << "\t" << bbd.d_status << "\n";
-      purgeAuthCaches(zone.toString() + "$");
+      purgeAuthCaches(zone.operator const DNSName&().toString() + "$");
       DNSSECKeeper::clearMetaCache(zone);
     }
     else
@@ -595,7 +665,7 @@ string Bind2Backend::DLDomStatusHandler(const vector<string>& parts, Utility::pi
   if (parts.size() > 1) {
     for (auto i = parts.begin() + 1; i < parts.end(); ++i) {
       BB2DomainInfo bbd;
-      if (safeGetBBDomainInfo(DNSName(*i), &bbd)) {
+      if (safeGetBBDomainInfo(ZoneName(*i), &bbd)) {
         ret << *i << ": " << (bbd.d_loaded ? "" : "[rejected]") << "\t" << bbd.d_status << "\n";
       }
       else {
@@ -657,7 +727,7 @@ string Bind2Backend::DLDomExtendedStatusHandler(const vector<string>& parts, Uti
   if (parts.size() > 1) {
     for (auto i = parts.begin() + 1; i < parts.end(); ++i) {
       BB2DomainInfo bbd;
-      if (safeGetBBDomainInfo(DNSName(*i), &bbd)) {
+      if (safeGetBBDomainInfo(ZoneName(*i), &bbd)) {
         printDomainExtendedStatus(ret, bbd);
       }
       else {
@@ -695,7 +765,7 @@ string Bind2Backend::DLAddDomainHandler(const vector<string>& parts, Utility::pi
   if (parts.size() < 3)
     return "ERROR: Domain name and zone filename are required";
 
-  DNSName domainname(parts[1]);
+  ZoneName domainname(parts[1]);
   const string& filename = parts[2];
   BB2DomainInfo bbd;
   if (safeGetBBDomainInfo(domainname, &bbd))
@@ -749,14 +819,14 @@ Bind2Backend::Bind2Backend(const string& suffix, bool loadZones)
     throw PDNSException("bind-hybrid and the zone cache currently interoperate badly. Please disable the zone cache or stop using bind-hybrid");
   }
 
-  d_transaction_id = 0;
+  d_transaction_id = UnknownDomainID;
   s_ignore_broken_records = mustDo("ignore-broken-records");
   d_upgradeContent = ::arg().mustDo("upgrade-unknown-types");
 
   if (!loadZones && d_hybrid)
     return;
 
-  std::lock_guard<std::mutex> l(s_startup_lock);
+  auto lock = std::scoped_lock(s_startup_lock);
 
   setupDNSSEC();
   if (s_first == 0) {
@@ -793,7 +863,7 @@ void Bind2Backend::reload()
   }
 }
 
-void Bind2Backend::fixupOrderAndAuth(std::shared_ptr<recordstorage_t>& records, const DNSName& zoneName, bool nsec3zone, const NSEC3PARAMRecordContent& ns3pr)
+void Bind2Backend::fixupOrderAndAuth(std::shared_ptr<recordstorage_t>& records, const ZoneName& zoneName, bool nsec3zone, const NSEC3PARAMRecordContent& ns3pr)
 {
   bool skip;
   DNSName shorter;
@@ -823,7 +893,7 @@ void Bind2Backend::fixupOrderAndAuth(std::shared_ptr<recordstorage_t>& records,
 
     if (!skip && nsec3zone && iter->qtype != QType::RRSIG && (iter->auth || (iter->qtype == QType::NS && (ns3pr.d_flags == 0u)) || (dssets.count(iter->qname) != 0u))) {
       Bind2DNSRecord bdr = *iter;
-      bdr.nsec3hash = toBase32Hex(hashQNameWithSalt(ns3pr, bdr.qname + zoneName));
+      bdr.nsec3hash = toBase32Hex(hashQNameWithSalt(ns3pr, bdr.qname + zoneName.operator const DNSName&()));
       records->replace(iter, bdr);
     }
 
@@ -831,7 +901,7 @@ void Bind2Backend::fixupOrderAndAuth(std::shared_ptr<recordstorage_t>& records,
   }
 }
 
-void Bind2Backend::doEmptyNonTerminals(std::shared_ptr<recordstorage_t>& records, const DNSName& zoneName, bool nsec3zone, const NSEC3PARAMRecordContent& ns3pr)
+void Bind2Backend::doEmptyNonTerminals(std::shared_ptr<recordstorage_t>& records, const ZoneName& zoneName, bool nsec3zone, const NSEC3PARAMRecordContent& ns3pr)
 {
   bool auth = false;
   DNSName shorter;
@@ -874,7 +944,7 @@ void Bind2Backend::doEmptyNonTerminals(std::shared_ptr<recordstorage_t>& records
   rr.ttl = 0;
   for (auto& nt : nonterm) {
     string hashed;
-    rr.qname = nt.first + zoneName;
+    rr.qname = nt.first + zoneName.operator const DNSName&();
     if (nsec3zone && nt.second)
       hashed = toBase32Hex(hashQNameWithSalt(ns3pr, rr.qname));
     insertRecord(records, zoneName, rr.qname, rr.qtype, rr.content, rr.ttl, hashed, &nt.second);
@@ -885,7 +955,7 @@ void Bind2Backend::doEmptyNonTerminals(std::shared_ptr<recordstorage_t>& records
 
 void Bind2Backend::loadConfig(string* status) // NOLINT(readability-function-cognitive-complexity) 13379 https://github.com/PowerDNS/pdns/issues/13379 Habbie: zone2sql.cc, bindbackend2.cc: reduce complexity
 {
-  static int domain_id = 1;
+  static domainid_t domain_id = 1;
 
   if (!getArg("config").empty()) {
     BindParser BP;
@@ -905,7 +975,8 @@ void Bind2Backend::loadConfig(string* status) // NOLINT(readability-function-cog
 
     g_log << Logger::Warning << d_logprefix << " Parsing " << domains.size() << " domain(s), will report when done" << endl;
 
-    set<DNSName> oldnames, newnames;
+    set<ZoneName> oldnames;
+    set<ZoneName> newnames;
     {
       auto state = s_state.read_lock();
       for (const BB2DomainInfo& bbd : *state) {
@@ -1019,12 +1090,12 @@ void Bind2Backend::loadConfig(string* status) // NOLINT(readability-function-cog
         safePutBBDomainInfo(bbd);
       }
     }
-    vector<DNSName> diff;
+    vector<ZoneName> diff;
 
     set_difference(oldnames.begin(), oldnames.end(), newnames.begin(), newnames.end(), back_inserter(diff));
     unsigned int remdomains = diff.size();
 
-    for (const DNSName& name : diff) {
+    for (const ZoneName& name : diff) {
       safeRemoveBBDomainInfo(name);
     }
 
@@ -1042,7 +1113,8 @@ void Bind2Backend::loadConfig(string* status) // NOLINT(readability-function-cog
   }
 }
 
-void Bind2Backend::queueReloadAndStore(unsigned int id)
+// NOLINTNEXTLINE(readability-identifier-length)
+void Bind2Backend::queueReloadAndStore(domainid_t id)
 {
   BB2DomainInfo bbold;
   try {
@@ -1110,7 +1182,8 @@ bool Bind2Backend::findBeforeAndAfterUnhashed(std::shared_ptr<const recordstorag
   return true;
 }
 
-bool Bind2Backend::getBeforeAndAfterNamesAbsolute(uint32_t id, const DNSName& qname, DNSName& unhashed, DNSName& before, DNSName& after)
+// NOLINTNEXTLINE(readability-identifier-length)
+bool Bind2Backend::getBeforeAndAfterNamesAbsolute(domainid_t id, const DNSName& qname, DNSName& unhashed, DNSName& before, DNSName& after)
 {
   BB2DomainInfo bbd;
   if (!safeGetBBDomainInfo(id, &bbd))
@@ -1142,32 +1215,32 @@ bool Bind2Backend::getBeforeAndAfterNamesAbsolute(uint32_t id, const DNSName& qn
         iter = --hashindex.end();
       before = DNSName(iter->nsec3hash);
     }
-    unhashed = iter->qname + bbd.d_name;
+    unhashed = iter->qname + bbd.d_name.operator const DNSName&();
 
     return true;
   }
 }
 
-void Bind2Backend::lookup(const QType& qtype, const DNSName& qname, int zoneId, DNSPacket* /* pkt_p */)
+void Bind2Backend::lookup(const QType& qtype, const DNSName& qname, domainid_t zoneId, DNSPacket* /* pkt_p */)
 {
   d_handle.reset();
 
   static bool mustlog = ::arg().mustDo("query-logging");
 
   bool found = false;
-  DNSName domain;
+  ZoneName domain;
   BB2DomainInfo bbd;
 
   if (mustlog)
     g_log << Logger::Warning << "Lookup for '" << qtype.toString() << "' of '" << qname << "' within zoneID " << zoneId << endl;
 
-  if (zoneId >= 0) {
+  if (zoneId != UnknownDomainID) {
     if ((found = (safeGetBBDomainInfo(zoneId, &bbd) && qname.isPartOf(bbd.d_name)))) {
       domain = std::move(bbd.d_name);
     }
   }
   else {
-    domain = qname;
+    domain = ZoneName(qname);
     do {
       found = safeGetBBDomainInfo(domain, &bbd);
     } while (!found && qtype != QType::SOA && domain.chopOff());
@@ -1274,7 +1347,8 @@ bool Bind2Backend::handle::get_normal(DNSResourceRecord& r)
   }
   DLOG(g_log << "Bind2Backend get() returning a rr with a " << QType(d_iter->qtype).getCode() << endl);
 
-  r.qname = qname.empty() ? domain : (qname + domain);
+  const DNSName& domainName(domain);
+  r.qname = qname.empty() ? domainName : (qname + domainName);
   r.domain_id = id;
   r.content = (d_iter)->content;
   //  r.domain_id=(d_iter)->domain_id;
@@ -1290,15 +1364,16 @@ bool Bind2Backend::handle::get_normal(DNSResourceRecord& r)
   return true;
 }
 
-bool Bind2Backend::list(const DNSName& /* target */, int id, bool /* include_disabled */)
+bool Bind2Backend::list(const ZoneName& /* target */, domainid_t domainId, bool /* include_disabled */)
 {
   BB2DomainInfo bbd;
 
-  if (!safeGetBBDomainInfo(id, &bbd))
+  if (!safeGetBBDomainInfo(domainId, &bbd)) {
     return false;
+  }
 
   d_handle.reset();
-  DLOG(g_log << "Bind2Backend constructing handle for list of " << id << endl);
+  DLOG(g_log << "Bind2Backend constructing handle for list of " << domainId << endl);
 
   if (!bbd.d_loaded) {
     throw PDNSException("zone was not loaded, perhaps because of: " + bbd.d_status);
@@ -1308,7 +1383,7 @@ bool Bind2Backend::list(const DNSName& /* target */, int id, bool /* include_dis
   d_handle.d_qname_iter = d_handle.d_records->begin();
   d_handle.d_qname_end = d_handle.d_records->end(); // iter now points to a vector of pointers to vector<BBResourceRecords>
 
-  d_handle.id = id;
+  d_handle.id = domainId;
   d_handle.domain = bbd.d_name;
   d_handle.d_list = true;
   return true;
@@ -1317,7 +1392,8 @@ bool Bind2Backend::list(const DNSName& /* target */, int id, bool /* include_dis
 bool Bind2Backend::handle::get_list(DNSResourceRecord& r)
 {
   if (d_qname_iter != d_qname_end) {
-    r.qname = d_qname_iter->qname.empty() ? domain : (d_qname_iter->qname + domain);
+    const DNSName& domainName(domain);
+    r.qname = d_qname_iter->qname.empty() ? domainName : (d_qname_iter->qname + domainName);
     r.domain_id = id;
     r.content = (d_qname_iter)->content;
     r.qtype = (d_qname_iter)->qtype;
@@ -1354,7 +1430,7 @@ bool Bind2Backend::autoPrimariesList(std::vector<AutoPrimary>& primaries)
   return true;
 }
 
-bool Bind2Backend::autoPrimaryBackend(const string& ip, const DNSName& /* domain */, const vector<DNSResourceRecord>& /* nsset */, string* /* nameserver */, string* account, DNSBackend** db)
+bool Bind2Backend::autoPrimaryBackend(const string& ipAddress, const ZoneName& /* domain */, const vector<DNSResourceRecord>& /* nsset */, string* /* nameserver */, string* account, DNSBackend** backend)
 {
   // Check whether we have a configfile available.
   if (getArg("autoprimary-config").empty())
@@ -1372,27 +1448,28 @@ bool Bind2Backend::autoPrimaryBackend(const string& ip, const DNSName& /* domain
   while (getline(c_if, line)) {
     std::istringstream ii(line);
     ii >> sip;
-    if (sip == ip) {
+    if (sip == ipAddress) {
       ii >> saccount;
       break;
     }
   }
   c_if.close();
 
-  if (sip != ip) // ip not found in authorization list - reject
+  if (sip != ipAddress) { // ip not found in authorization list - reject
     return false;
+  }
 
   // ip authorized as autoprimary - accept
-  *db = this;
+  *backend = this;
   if (saccount.length() > 0)
     *account = saccount.c_str();
 
   return true;
 }
 
-BB2DomainInfo Bind2Backend::createDomainEntry(const DNSName& domain, const string& filename)
+BB2DomainInfo Bind2Backend::createDomainEntry(const ZoneName& domain, const string& filename)
 {
-  int newid = 1;
+  domainid_t newid = 1;
   { // Find a free zone id nr.
     auto state = s_state.read_lock();
     if (!state->empty()) {
@@ -1413,16 +1490,16 @@ BB2DomainInfo Bind2Backend::createDomainEntry(const DNSName& domain, const strin
   return bbd;
 }
 
-bool Bind2Backend::createSecondaryDomain(const string& ip, const DNSName& domain, const string& /* nameserver */, const string& account)
+bool Bind2Backend::createSecondaryDomain(const string& ipAddress, const ZoneName& domain, const string& /* nameserver */, const string& account)
 {
   string filename = getArg("autoprimary-destdir") + '/' + domain.toStringNoDot();
 
   g_log << Logger::Warning << d_logprefix
         << " Writing bind config zone statement for superslave zone '" << domain
-        << "' from autoprimary " << ip << endl;
+        << "' from autoprimary " << ipAddress << endl;
 
   {
-    std::lock_guard<std::mutex> l2(s_autosecondary_config_lock);
+    auto lock = std::scoped_lock(s_autosecondary_config_lock);
 
     ofstream c_of(getArg("autoprimary-config").c_str(), std::ios::app);
     if (!c_of) {
@@ -1435,14 +1512,14 @@ bool Bind2Backend::createSecondaryDomain(const string& ip, const DNSName& domain
     c_of << "zone \"" << domain.toStringNoDot() << "\" {" << endl;
     c_of << "\ttype secondary;" << endl;
     c_of << "\tfile \"" << filename << "\";" << endl;
-    c_of << "\tprimaries { " << ip << "; };" << endl;
+    c_of << "\tprimaries { " << ipAddress << "; };" << endl;
     c_of << "};" << endl;
     c_of.close();
   }
 
   BB2DomainInfo bbd = createDomainEntry(domain, filename);
   bbd.d_kind = DomainInfo::Secondary;
-  bbd.d_primaries.push_back(ComboAddress(ip, 53));
+  bbd.d_primaries.emplace_back(ComboAddress(ipAddress, 53));
   bbd.setCtime();
   safePutBBDomainInfo(bbd);
 
@@ -1472,7 +1549,8 @@ bool Bind2Backend::searchRecords(const string& pattern, size_t maxResults, vecto
       shared_ptr<const recordstorage_t> rhandle = h.d_records.get();
 
       for (recordstorage_t::const_iterator ri = rhandle->begin(); result.size() < maxResults && ri != rhandle->end(); ri++) {
-        DNSName name = ri->qname.empty() ? i.d_name : (ri->qname + i.d_name);
+        const DNSName& domainName(i.d_name);
+        DNSName name = ri->qname.empty() ? domainName : (ri->qname + domainName);
         if (sm.match(name) || sm.match(ri->content)) {
           DNSResourceRecord r;
           r.qname = std::move(name);
