@@ -1347,7 +1347,7 @@ yyreduce:
   case 11: /* global_zone_command: ZONETOK quotedname zone_block  */
 #line 134 "bindparser.yy"
         {
-		s_di.name=DNSName(yyvsp[-1]);
+		s_di.name=ZoneName(yyvsp[-1]);
 		free(yyvsp[-1]);
 		parent->commit(s_di);
 		s_di.clear();
@@ -1358,7 +1358,7 @@ yyreduce:
   case 12: /* global_zone_command: ZONETOK quotedname AWORD zone_block  */
 #line 142 "bindparser.yy"
         {
-	        s_di.name=DNSName(yyvsp[-2]);
+	        s_di.name=ZoneName(yyvsp[-2]);
 		free(yyvsp[-2]);
 		parent->commit(s_di);
 		s_di.clear();
